#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.icons.font_awesome
      @file: form_icons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome

class FormIcons(Awesome):
    """
        Form UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    ARROW_LEFT = '\uF060'      # 
    ARROW_RIGHT = '\uF061'     # 
    ARROW_UP = '\uF062'        # 
    ARROW_DOWN = '\uF063'      # 
    SELECTOR = '\uFC32'        # ﰲ

    CHECK = '\uF00C'           # 
    ERROR = '\uf057'           # 
    CHECK_CIRCLE = '\uF058'    # 
    UNCHECK_CIRCLE = '\uF111'  # 
    CHECK_SQUARE = '\uF14A'    # 
    UNCHECK_SQUARE = '\uF0C8'  # 
    ON = '\uF205'              # 
    OFF = '\uF204'             # 
    MARKED = '\uF634'          # 
    UNMARKED = '\uF630'        # 
    CLEAR = '\uF5E1'           # 
    REFRESH = '\uF01E'         # 
    FILTER  = '\uF0B0'         # 
    ATTACH = '\uF0C6'          # 

    HIDDEN = '\uF070'          # 
    VISIBLE = '\uF06E'         # 
    LOCKED = '\uF023'          # 
    UNLOCKED = '\uF09C'        # 
    EDITABLE = '\uF044'        # 
    MASKED = '\uF0CE'          # 
    SELECTABLE = '\uF150'      # 
    DESELECT = '\uF657'        # 
    DELETE = '\uF014'          # 
    EDIT = '\uF040'            # 
    PLUS = '\uF067'            # 
    MINUS = '\uF068'           # 
    # @formatter:on


if __name__ == '__main__':
    FormIcons.demo_icons()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.table
      @file: table_renderer.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import sys
from abc import ABC
from typing import List, Optional

from hspylib.core.tools.preconditions import check_argument
from hspylib.core.tools.text_tools import elide_text, justified_center, justified_left, justified_right


class TableRenderer:
    """TODO"""

    # pylint: disable=too-few-public-methods
    class TextAlignment(ABC):
        """
        Table cell text justification helper.
        """
        LEFT = justified_left
        CENTER = justified_center
        RIGHT = justified_right

    def __init__(
        self,
        table_headers: List[str],
        table_data: Optional[iter],
        table_caption: str = None):
        """
        :param table_headers: table headers to be displayed.
        :param table_data: table record set with the selected rows.
        :param table_caption: table caption to be displayed.
        """

        self.headers = table_headers
        self.rows = table_data if table_data else []
        self.caption = table_caption
        self.header_alignment = TableRenderer.TextAlignment.CENTER
        self.cell_alignment = TableRenderer.TextAlignment.LEFT
        self.min_column_size = 6
        if self.rows:
            check_argument(
                len(min(self.rows, key=len)) == len(self.headers),
                'Headers and Columns must have the same size: {} vs {}',
                len(min(self.rows, key=len)), len(self.headers))
        self.column_sizes = [max(self.min_column_size, len(header)) for header in self.headers]
        self.indexes = range(0, len(self.column_sizes))

    def set_header_alignment(self, alignment: TextAlignment) -> None:
        """
        Set table header justification.
        :param alignment: table header text alignment function.
        :return:
        """
        self.header_alignment = alignment

    def set_cell_alignment(self, alignment: TextAlignment) -> None:
        """
        Set table header justification.
        :param alignment: table cell text alignment function.
        :return:
        """
        self.cell_alignment = alignment

    def set_min_column_size(self, size: int) -> None:
        """
        Set table header justification.
        :param size: minimum table cell size.
        :return:
        """
        self.min_column_size = size

    def adjust_sizes_by_largest_cell(self) -> None:
        """
        Render table based on the maximum size of all cell data.
        :return: None
        """
        for row in self.rows:
            for idx, dummy in enumerate(row):
                self.column_sizes[idx] = max(self.column_sizes[idx], len(str(row[idx])))

    def set_fixed_cell_size(self, width: int) -> None:
        """
        Render table based on a fixed size for all cell data.
        :return: None
        """
        for row in self.rows:
            for idx in range(0, len(row)):
                self.column_sizes[idx] = max(width, self.min_column_size)

    def set_cell_sizes(self, cell_sizes: List[int]) -> None:
        """
        Render table based on a list of fixed sizes.
        :return: None
        """
        check_argument(len(min(self.rows, key=len)) == len(cell_sizes),
                       'Sizes and Columns must have the same size: {} vs {}', len(min(self.rows, key=len)),
                       len(cell_sizes))
        for row in self.rows:
            for idx in range(0, len(row)):
                self.column_sizes[idx] = max(cell_sizes[idx], self.min_column_size)

    def render(self, file=sys.stdout) -> None:
        """
        Render table based on the maximum size of a column header.
        :param file: a file-like object (stream); defaults to the current sys.stdout.
        :return: None
        """
        header_cols = self._join_header_columns()
        data_cols = self._join_data_columns()
        table_borders = '+' + ''.join((('-' * (self.column_sizes[idx] + 2) + '+') for idx in self.indexes))
        self._print_table(table_borders, header_cols, data_cols, file)

    def _join_header_columns(self) -> list:
        """TODO"""
        cols = [self.header_alignment(self._header_text(idx), self.column_sizes[idx]) for idx in self.indexes]
        return ['| ' + ' | '.join(cols) + ' |']

    # pylint: disable=consider-using-f-string
    def _join_data_columns(self) -> list:
        """TODO"""
        return [
            '| ' + ''.join(
                '%s | ' % self.cell_alignment(self._cell_text(row, idx), self.column_sizes[idx]) for idx in self.indexes
            ) for row in self.rows
        ]

    def _header_text(self, idx: int) -> str:
        """TODO"""
        return elide_text(self.headers[idx], self._cell_size(idx))

    def _cell_text(self, row: tuple, idx: int) -> str:
        """TODO"""
        return elide_text(str(row[idx]), self._cell_size(idx))

    def _cell_size(self, idx: int) -> int:
        """TODO"""
        return self.column_sizes[idx]

    def _print_table(
        self,
        table_line: str,
        header_cols: List[str],
        data_cols: List[str],
        file=sys.stdout) -> None:
        """TODO"""

        if self.caption:
            print(table_line, file=file)
            print('| ' + elide_text(self.caption, len(table_line) - 4)
                  .center(len(header_cols[0]) - 4, ' ') + ' |', file=file)
        print(table_line, file=file)
        print('\n'.join(header_cols), file=file)
        print(table_line, file=file)
        print(
            '\n'.join(data_cols) if data_cols
            else '| ' + '<empty>'.center(len(table_line) - 4, ' ') + ' |', file=file
        )
        print(table_line, file=file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.tools
      @file: text_tools.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import os
import random
import re
from typing import Any, List, Optional, Tuple

from hspylib.core.tools.commons import get_or_default


def elide_text(text: str, width: int, elide_with: str = '...') -> str:
    """ Return a copy of the string eliding the text case the string is bigger than the specified length. """
    return text if len(text) <= width else text[:width - len(elide_with)] + elide_with


def cut(text: str, field: int, separator: str = ' ') -> Tuple[Optional[str], tuple]:
    """ Return a new string cut out from the given text. """
    result = tuple(re.split(rf'{separator}+', text))
    return get_or_default(result, field), result[:field]


def random_string(choices: List[str], length: int, weights: List[int] = None) -> str:
    """ Return a new random string matching choices and length. """
    return ''.join(random.choices(choices, weights or [1] * len(choices), k=length))


def justified_left(text: str, width: int, fill: str = ' ') -> str:
    """ Return a copy of the string justified left. """
    return text.ljust(width, fill)


def justified_center(text: str, width: int, fill: str = ' ') -> str:
    """ Return a copy of the string justified center. """
    return text.center(width, fill)


def justified_right(text: str, width: int, fill: str = ' ') -> str:
    """ Return a copy of the string justified right. """
    return text.rjust(width, fill)


def uppercase(text: str) -> str:
    """ Return a copy of the string converted to upper case. """
    return text.upper()


def lowercase(text: str) -> str:
    """ Return a copy of the string converted to lower case. """
    return text.lower()


def camelcase(text: str, separator: str = ' |-|_', upper: bool = False) -> str:
    """ Return a copy of the string converted to camel case.
    Ref:https://en.wikipedia.org/wiki/Letter_case :: Camel case"""
    s = re.sub(rf"({separator})+", " ", text).title().replace(" ", "")
    fnc = getattr(s[0], 'lower' if not upper else 'upper')
    return ''.join([fnc(), s[1:]])


def snakecase(text: str, separator: str = '-', screaming: bool = False) -> str:
    """ Return a copy of the string converted to snake case.
    Ref:https://en.wikipedia.org/wiki/Letter_case :: Snake case
    """
    s = re.sub('([A-Z][a-z]+)', r' \1', re.sub('([A-Z]+)', r' \1', text.replace(separator, ' ')))
    string = '_'.join(s.split())
    fnc = getattr(string, 'lower' if not screaming else 'upper')
    return fnc()


def kebabcase(text: str, separator: str = ' |-|_', train: bool = False) -> str:
    """ Return a copy of the string converted to kebab case.
    Ref:https://en.wikipedia.org/wiki/Letter_case :: Kebab case
    """
    s = re.sub('([A-Z][a-z]+)', r' \1', re.sub('([A-Z]+)', r' \1', text.replace(separator, ' ')))
    string = '-'.join(s.split())
    fnc = getattr(string, 'lower' if not train else 'upper')
    return fnc()


def titlecase(text: str, separator: str = ' |-|_', skip_length: int = 0) -> str:
    """ Return a copy of the string converted to title case.
    Ref:https://en.wikipedia.org/wiki/Letter_case :: Title case
    """
    s = re.sub(rf"({separator})+", " ", text)
    s = ' '.join([word.title() if len(word) > skip_length else word.lower() for word in s.split(' ')])
    return s


def strip_escapes(text: str) -> str:
    """ Return a copy of the string stripping out all ansi escape 'ESC[' codes from it.
    Ref:https://stackoverflow.com/questions/14693701/how-can-i-remove-the-ansi-escape-sequences-from-a-string-in-python
    """
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    return ansi_escape.sub('', text)


def strip_linebreaks(text: str, re_exp: str = r'(\n|\r|\n\r)+') -> str:
    """ Return a copy of the string stripping out all line breaks from it. """
    return re.sub(re.compile(rf'{re_exp}'), '', text)


def strip_extra_spaces(text: str, re_exp: str = r'\s+', trim: bool = False) -> str:
    """ Return a copy of the string stripping out all extra spaces 2+ from it. """
    s = re.sub(re.compile(rf'{re_exp}'), ' ', text)
    return s if not trim else s.strip()


def json_stringify(json_string: str) -> str:
    """ Return a copy of the json string stripping any line breaks or formatting from it and also quoting any existing
    double quotes. """
    return strip_extra_spaces(strip_linebreaks(json_string)).replace('"', '\\"')


def eol(current_index: int, split_len: int, line_sep: str = os.linesep, word_sep: str = ' ') -> str:
    """Give the line separator character (hit) or an empty string (miss) according to the splitting length and
    current index. """
    return line_sep if (current_index + 1) % split_len == 0 else word_sep


def quote(value: Any) -> str:
    """Quote or double quote the value according to the value type. """
    return str(value) if not isinstance(value, str) \
        else f'"{value}"' if value.startswith('\'') and value.endswith('\'') else f"'{value}'"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.tools
      @file: validator.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import re
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Tuple, Type

from hspylib.core.tools.constants import DATE_TIME_FORMAT


class Validator(ABC):

    @staticmethod
    def assert_valid(errors: list, validation: Tuple[bool, str], throw_if_invalid: bool = False) -> None:
        if not validation[0]:
            if throw_if_invalid:
                raise AssertionError(validation[1])
            errors.append(validation[1])

    @staticmethod
    def is_not_blank(input_string: str, min_length: int = 0) -> bool:
        return input_string and len(input_string) >= min_length

    @staticmethod
    def matches(
        input_string: str,
        regex_pattern: str) -> bool:

        return bool(re.match(regex_pattern, input_string))

    @staticmethod
    def is_integer(
        number: str,
        min_value: int = 0,
        max_value: int = 65535) -> bool:

        return number.isdigit() and min_value <= int(number) <= max_value

    @staticmethod
    def is_float(
        number: str,
        min_value: float = 0,
        max_value: float = 65535) -> bool:

        return number.isdecimal() and min_value <= float(number) <= max_value

    @staticmethod
    def is_enum(
        name: str,
        enum_type: Type) -> bool:

        return name.upper() in enum_type.__dict__

    @staticmethod
    def is_date(
        date_text: str,
        fmt: str = DATE_TIME_FORMAT) -> bool:

        try:
            datetime.strptime(date_text, fmt)
            return True
        except ValueError:
            return False

    @staticmethod
    def has_no_nulls(*args) -> bool:
        return all(opt is not None for opt in args)

    @staticmethod
    def in_range(array: list, start: int, end: int) -> bool:
        return \
            list is not None \
            and 0 <= start < len(array) \
            and 0 <= start <= end < len(array)

    @abstractmethod
    def __call__(self, *args, **kwargs) -> bool:
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: hlabel.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
from typing import Optional

from PyQt5.QtCore import pyqtSignal, Qt
from PyQt5.QtGui import QFontMetrics, QMouseEvent, QResizeEvent, QTextDocument
from PyQt5.QtWidgets import QLabel, QSizePolicy, QWidget


class HLabel(QLabel):
    """TODO"""

    clicked = pyqtSignal()
    elisionChanged = pyqtSignal()

    def __init__(self, parent: Optional[QWidget]):
        super().__init__(parent)
        self._clickable = False
        self._elidable = True
        self._dynamic_tooltip = False
        self._content = self.text()
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)

    def elidable(self) -> bool:
        return self._elidable

    def set_elidable(self, elidable: bool) -> None:
        self._elidable = elidable

    def clickable(self) -> bool:
        return self._clickable

    def set_clickable(self, clickable: bool) -> None:
        self._clickable = clickable
        self.setCursor(Qt.PointingHandCursor if clickable else Qt.ArrowCursor)

    def dynamic_tooltip(self) -> bool:
        return self._dynamic_tooltip

    def set_dynamic_tooltip(self, dynamic_tooltip: bool) -> None:
        self._dynamic_tooltip = dynamic_tooltip

    def mousePressEvent(self, ev: QMouseEvent) -> None:  # pylint: disable=unused-argument
        if self._clickable:
            self.clicked.emit()

    def setText(self, text: str):
        self._content = text
        if not self.toolTip() or self._dynamic_tooltip:
            self.setToolTip(text)
        if self._elidable:
            metrics = QFontMetrics(self.font())
            max_length = int(self.width() / metrics.maxWidth())
            doc = QTextDocument()
            doc.setHtml(text)
            plain_text = doc.toPlainText()
            if len(plain_text) > max_length:
                self.elisionChanged.emit()
                elided_last_line = metrics.elidedText(text, Qt.ElideRight, self.width())
                super().setText(elided_last_line)
                return
        super().setText(text)

    def resizeEvent(self, event: QResizeEvent) -> None:
        if self._content and self._content != self.text():
            self.setText(self._content)
        super().resizeEvent(event)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: main.crud.db.firebase
      @file: firebase_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import json
import logging as log
import uuid
from abc import abstractmethod
from typing import Optional

from requests.exceptions import HTTPError
from requests.structures import CaseInsensitiveDict

from hspylib.core.crud.crud_entity import CrudEntity
from hspylib.core.crud.crud_repository import CrudRepository
from hspylib.core.crud.db.firebase.firebase_config import FirebaseConfig
from hspylib.core.enums.http_code import HttpCode
from hspylib.core.tools.preconditions import check_not_none, check_state
from hspylib.modules.fetch.fetch import delete, get, put


class FirebaseRepository(CrudRepository):
    """Implementation of a data access layer for a Firebase persistence store."""

    def __init__(self):
        self.payload = None
        self.config = FirebaseConfig()

    def __str__(self):
        return str(self.payload)

    def to_list(self, json_string: str, filters: CaseInsensitiveDict = None) -> list:
        """Return filtered entries from the json_string as a list"""
        ret_list = []
        for value in json.loads(json_string).values():
            if not filters or all(k in value and value[k] == v for k, v in filters.items()):
                ret_list.append(self.row_to_entity(value))
        return ret_list

    def insert(self, entity: CrudEntity) -> None:
        """Saves the given entity at the Firebase store"""
        entity.uuid = entity.uuid if entity.uuid is not None else str(uuid.uuid4())
        url = f'{self.config.base_url()}/{entity.uuid}.json'
        payload = entity.to_json()
        log.debug("Inserting firebase entry: %s into: %s", entity, url)
        response = put(url, payload)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError(f'{response.status_code} - Unable to put into={url} with json_string={payload}')

    def update(self, entity: CrudEntity) -> None:
        """Updates the given entity at the Firebase store"""
        url = f'{self.config.base_url()}/{entity.uuid}.json'
        payload = entity.to_json()
        log.debug('Updating firebase entry: %s into: %s', entity, url)
        response = put(url, payload)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError(f'{response.status_code} - Unable to put into={url} with json_string={payload}')

    def delete(self, entity: CrudEntity) -> None:
        """Deletes the given entity from the Firebase store"""
        url = f'{self.config.base_url()}/{entity.uuid}.json'
        log.debug('Deleting firebase entry: %s into: %s', entity, url)
        response = delete(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError(f'{response.status_code} - Unable to delete from={url}')

    def find_all(self, filters: CaseInsensitiveDict = None) -> Optional[list]:
        """Return filtered entries from the Firebase store"""
        url = f'{self.config.base_url()}.json?orderBy="$key"'
        log.debug('Fetching firebase entries from %s', url)
        response = get(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError(f'{response.status_code} - Unable to get from={url}')

        return self.to_list(response.body, filters) if response.body else []

    def find_by_id(self, entity_id: str) -> Optional[CrudEntity]:
        """Return the entry specified by ID from the Firebase store, None if no such entry is found."""
        url = f'{self.config.base_url()}.json?orderBy="$key"&equalTo="{entity_id}"'
        log.debug('Fetching firebase entry entity_id=%s from %s', entity_id, url)
        response = get(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError(f'{response.status_code} - Unable to get from={url}')
        result = self.to_list(response.body) if response.body else []
        check_state(len(result) <= 1, "Multiple results found with entity_id={}", entity_id)

        return result[0] if len(result) > 0 else None

    @abstractmethod
    def row_to_entity(self, row: dict) -> CrudEntity:
        """TODO"""
        pass

    @abstractmethod
    def database_name(self) -> str:
        """TODO"""
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.web
      @file: soap_utils.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import logging as log
from typing import Optional

from requests.structures import CaseInsensitiveDict

from hspylib.core.enums.http_method import HttpMethod
from hspylib.modules.fetch.fetch import fetch
from hspylib.modules.fetch.http_response import HttpResponse


def soap_call(
    url: str,
    method: HttpMethod,
    data: str,
    headers: Optional[CaseInsensitiveDict]) -> Optional[HttpResponse]:
    """TODO"""

    all_headers = {} if not headers else headers
    all_headers.update({
        "Content-Type": "text/xml",
        "Accept": "*/*"
    })
    log.info("Processing SOAP %s {method} -> %s \n%s", all_headers, url, data if data else '<None>')
    response = fetch(url=url, method=method, headers=all_headers, body=data)
    log.info("Response <=  Status: %s  Payload: %s", response.status_code, response.body if response.body else '<None>')

    return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.extra.minput
      @file: input_type.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
from hspylib.core.enums.enumeration import Enumeration

class InputType(Enumeration):
    """TODO"""

    TEXT = 'text'
    PASSWORD = 'password'
    CHECKBOX = 'checkbox'
    SELECT = 'select'
    MASKED = 'masked'

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return str(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   IP Message Sender. Sends TCP/UDP messages (multi-threaded).

   @project: HSPyLib
   @package: main.addons.widman.widgets
      @file: widget_send_msg.py
   @created: Thu, 26 Aug 2017
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import os
import signal
import socket
import threading
from textwrap import dedent
from time import sleep
from typing import List

from hspylib.addons.widman.widget import Widget
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.exception.exceptions import WidgetExecutionError
from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.application.argparse.argument_parser import HSArgumentParser
from hspylib.modules.cli.icons.font_awesome.widget_icons import WidgetIcons
from hspylib.modules.cli.tui.extra.minput.input_validator import InputValidator
from hspylib.modules.cli.tui.extra.minput.minput import MenuInput, minput
from hspylib.modules.cli.tui.menu.menu_utils import MenuUtils


class WidgetSendMsg(Widget):
    """HSPyLib Widget to send TCP/UDP messages (multi-threaded)"""
    WIDGET_ICON = WidgetIcons.NETWORK
    WIDGET_NAME = "SendMsg"
    TOOLTIP = "IP Message Sender. Sends TCP/UDP messages (multi-threaded)"
    VERSION = (0, 3, 0)
    MAX_THREADS = 1000
    NET_TYPE_UDP = 'UDP'
    NET_TYPE_TCP = 'TCP'

    # Help message to be displayed by the application.
    USAGE = dedent(f"""Usage: SendMsg [options]

      Options:
        +n, ++net_type   <network_type>     : The network type to be used. Either udp or tcp ( default is tcp ).
        +p, ++port       <port_num>         : The port number [1-65535] ( default is 12345).
        +a, ++address    <host_address>     : The address of the datagram receiver ( default is 127.0.0.1 ).
        +k, ++packets    <num_packets>      : The number of max datagrams to be send. If zero is specified, then the app
                                              is going to send indefinitely ( default is 100 ).
        +i, ++interval   <interval_MS>      : The interval in seconds between each datagram ( default is 1 Second ).
        +t, ++threads    <threads_num>      : Number of threads [1-{MAX_THREADS}] to be opened to send simultaneously
                                              ( default is 1 ).
        +m, ++message    <message/filename> : The message to be sent. If the message matches a filename, then the file
                                              contents sent instead.

        E.g:. send-msg.py +n tcp +m "Hello" +p 12345 +a 0.0.0.0 +k 100 +i 500 +t 2
    """)

    def __init__(self):
        super().__init__(
            self.WIDGET_ICON,
            self.WIDGET_NAME,
            self.TOOLTIP,
            self.USAGE,
            self.VERSION)

        self.is_alive = True
        self.net_type = None
        self.host = None
        self.packets = None
        self.interval = None
        self.threads = None
        self.message = None
        self.args = None
        self.socket = None
        self.counter = 1

    def execute(self, args: List[str] = None) -> ExitCode:
        signal.signal(signal.SIGINT, self.cleanup)
        signal.signal(signal.SIGTERM, self.cleanup)

        if args and args[0] in ['-h', '--help']:
            sysout(self.usage())
            return ExitCode.SUCCESS
        if args and args[0] in ['-v', '--version']:
            sysout(self.version())
            return ExitCode.SUCCESS

        if args and not self._parse_args(args):
            return ExitCode.ERROR
        if not args and not self._read_args():
            return ExitCode.ERROR
        if not args and not self.args:
            return ExitCode.ERROR

        self.net_type = self.args.net_type or self.NET_TYPE_TCP
        self.host = (self.args.address or '127.0.0.1', self.args.port or 12345)
        self.packets = self.args.packets or 100
        self.interval = self.args.interval or 1
        self.threads = self.args.threads or 1

        if self.args.message and os.path.isfile(self.args.message):
            file_size = os.stat(self.args.message).st_size
            sysout(f"Reading contents from file: {self.args.message} ({file_size}) [Bs] instead")
            with open(self.args.message, 'r', encoding='utf-8') as f_msg:
                self.message = f_msg.read()
        else:
            self.message = self.args.message or f"This is a {self.args.net_type} test %(count)"

        self._start_send()

        MenuUtils.wait_enter()

        return ExitCode.SUCCESS

    def cleanup(self) -> None:
        sysout('Terminating threads%NC%')
        self.is_alive = False
        if self.net_type == self.NET_TYPE_TCP:
            sysout('Closing TCP connection')
            self.socket.close()

    def _read_args(self) -> bool:
        """When no input is provided (e.g:. when executed from dashboard). Prompt the user for the info. """
        # @formatter:off
        form_fields = MenuInput.builder() \
            .field() \
                .label('Net Type') \
                .itype('select') \
                .value(f"{self.NET_TYPE_TCP}|{self.NET_TYPE_UDP}") \
                .build() \
            .field() \
                .label('Address') \
                .validator(InputValidator.anything()) \
                .value('127.0.0.1') \
                .build() \
            .field() \
                .label('Port') \
                .validator(InputValidator.numbers()) \
                .min_max_length(2, 5) \
                .value(12345) \
                .build() \
            .field() \
                .label('Packets') \
                .validator(InputValidator.numbers()) \
                .min_max_length(1, 4) \
                .value(100) \
                .build() \
            .field() \
                .label('Interval') \
                .validator(InputValidator.numbers()) \
                .min_max_length(1, 4) \
                .value(1) \
                .build() \
            .field() \
                .label('Threads') \
                .validator(InputValidator.numbers()) \
                .min_max_length(1, 4) \
                .value(1) \
                .build() \
            .field() \
                .label('Message') \
                .validator(InputValidator.anything()) \
                .min_max_length(1, 40) \
                .value('This is a test') \
                .build() \
            .build()
        # @formatter:on
        self.args = minput(form_fields)

        return len(self.args.__dict__) > 1 if self.args else False

    def _parse_args(self, args: List[str]):
        """When arguments are passed from the command line, parse them"""
        parser = HSArgumentParser(
            prog='sendmsg', prefix_chars="+", description='Sends TCP/UDP messages (multi-threaded)')
        parser.add_argument(
            '+n', '++net-type', action='store', type=str, choices=['udp', 'tcp'], default='tcp', required=False,
            help='The network type to be used. Either udp or tcp ( default is tcp )')
        parser.add_argument(
            '+a', '++address', action='store', type=str, default='127.0.0.1', required=False,
            help='The address of the datagram receiver ( default is 127.0.0.1 )')
        parser.add_argument(
            '+p', '++port', action='store', type=int, default=12345, required=False,
            help='The port number [1-65535] ( default is 12345)')
        parser.add_argument(
            '+k', '++packets', action='store', type=int, default=100, required=False,
            help='The number of max datagrams to be send. If zero is specified, then the app '
                 'is going to send indefinitely ( default is 100 ).')
        parser.add_argument(
            '+i', '++interval', action='store', type=float, default=1, required=False,
            help='The interval in seconds between each datagram ( default is 1 Second )')
        parser.add_argument(
            '+t', '++threads', action='store', type=int, default=1, required=False,
            help=f'Number of threads [1-{self.MAX_THREADS}] to be opened to send simultaneously ( default is 1 )')
        parser.add_argument(
            '+m', '++message', action='store', type=str, required=False,
            help='The message to be sent. If the message matches a filename, then the file contents sent instead')
        self.args = parser.parse_args(args)

        return bool(self.args)

    def _init_sockets(self) -> None:
        """Initialize sockets"""
        if self.net_type == self.NET_TYPE_UDP:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        else:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                self.socket.connect(self.host)
                sysout(f"Successfully connected to {self.host}")
            except socket.error as err:
                raise WidgetExecutionError('Unable to initialize sockets') from err

    def _start_send(self) -> None:
        """Start sending packets"""
        thread_relief = 0.05
        self._init_sockets()
        sysout(f"\n%ORANGE%Start sending {self.packets} "
               f"{self.net_type.upper()} packet(s) "
               f"every {self.interval} second(s) to {self.host} using {self.threads} thread(s)")
        threads_num = threading.active_count()

        for thread_num in range(1, int(self.threads) + 1):
            tr = threading.Thread(target=self._send_packet, args=(thread_num,))
            tr.daemon = True
            tr.start()
            sleep(thread_relief)

        while self.is_alive and threading.active_count() > threads_num:
            sleep(2 * thread_relief)

    def _send_packet(self, thread_num: int) -> None:
        """Send a packet"""
        lock = threading.Lock()

        while self.is_alive and self.packets <= 0 or self.counter <= self.packets:
            message = self.message.replace('%(count)', str(self.counter))
            length = len(message)
            sysout(f"%BLUE%[Thread-{thread_num:d}] "
                   f"%GREEN%Sending \"{message:s}\" ({length:d}) bytes, "
                   f"Pkt = {self.counter:>d}/{self.packets:>d} %NC%...")
            if self.net_type == self.NET_TYPE_UDP:
                self.socket.sendto(message.encode(), self.host)
            else:
                try:
                    self.socket.sendall((message + '\n').encode())
                    with lock:
                        self.counter += 1
                except socket.error as err:
                    raise WidgetExecutionError('Unable to send packet') from err
            sleep(self.interval)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: hspylib
   @package: hspylib.main.hspylib.core.tools
      @file: collection_filter.py
   @created: Wed, 8 Jun 2022
    @author: "<B>H</B>ugo <B>S</B>aporetti <B>J</B>unior")"
      @site: "https://github.com/yorevs/hspylib")
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
from typing import get_args, List, Set, TypeVar, Union

from hspylib.core.enums.enumeration import Enumeration
from hspylib.core.tools.preconditions import check_argument
from hspylib.core.tools.text_tools import quote

T = TypeVar('T')
FILTER_VALUE = TypeVar('FILTER_VALUE', int, str, bool, float)


class FilterConditions(Enumeration):
    """TODO"""

    # @formatter:off
    LESS_THAN                   =      '<', Union[int, float]
    LESS_THAN_OR_EQUALS_TO      =     '<=', Union[int, float]
    GREATER_THAN                =      '>', Union[int, float]
    GREATER_THAN_OR_EQUALS_TO   =     '>=', Union[int, float]
    EQUALS_TO                   =     '==', Union[str, int, bool, float]
    DIFFERENT_FROM              =     '!=', Union[str, int, bool, float]
    CONTAINS                    =     'in', Union[str]
    DOES_NOT_CONTAIN            = 'not in', Union[str]
    IS                          =     '==', Union[bool]
    IS_NOT                      =     '!=', Union[bool]
    # @formatter:on

    def __str__(self):
        return self.name.lower().replace('_', ' ')

    def __repr__(self):
        return str(self)

    def matches(self, param_value: FILTER_VALUE, value: FILTER_VALUE) -> bool:
        """TODO"""
        operator = self.value[0]
        if self.name in ['CONTAINS', 'DOES_NOT_CONTAIN']:
            expression = f'{quote(value)} {operator} {quote(param_value)}'
        else:
            expression = f'{quote(param_value)} {operator} {quote(value)}'
        return self._has_type(value) and eval(expression)  # pylint: disable=eval-used

    def _has_type(self, value: FILTER_VALUE) -> bool:
        """TODO"""
        try:
            return isinstance(value, self.value[1])
        except TypeError:
            return isinstance(value, get_args(self.value[1]))


class ElementFilter:

    def __init__(
        self, name: str,
        el_name: str,
        condition: 'FilterConditions',
        el_value: FILTER_VALUE):
        self.name = name
        self.el_name = el_name
        self.condition = condition
        self.el_value = el_value

    def __str__(self):
        return f'{quote(self.el_name)} {self.condition} {quote(self.el_value)}'

    def __repr__(self):
        return str(self)

    def matches(self, element: T) -> bool:
        """TODO"""
        try:
            entry = element if isinstance(element, dict) else element.__dict__
            return self.el_name in entry and self.condition.matches(entry[self.el_name], self.el_value)
        except (NameError, TypeError, AttributeError):
            return False


class CollectionFilter:
    """TODO"""

    def __init__(self):
        self._filters: Set[ElementFilter] = set()

    def __str__(self):
        if len(self._filters) > 0:
            return ' '.join([f"{'AND ' if i > 0 else ''}{str(f)}" for i, f in enumerate(self._filters)])

        return 'No filters applied'

    def __repr__(self):
        return str(self)

    def __iter__(self):
        """Returns the Iterator object"""
        return self._filters.__iter__()

    def apply_filter(
        self,
        name: str,
        el_name: str,
        condition: 'FilterConditions',
        el_value: Union[int, str, bool, float]) -> None:
        """TODO"""

        check_argument(not any(f.name == name for f in self._filters),
                       f'Filter {name} already exists!')
        # Avoid applying the same filter conditions
        if not any(
            f.el_name == el_name
            and f.condition == condition
            and f.el_value == el_value for f in self._filters):
            self._filters.add(ElementFilter(name, el_name, condition, el_value))

    def clear(self) -> None:
        """TODO"""
        self._filters.clear()

    def size(self) -> int:
        """TODO"""
        return len(self._filters)

    def discard(self, name: str):
        """TODO"""
        element = next((e for e in self._filters if e.name == name), None)
        self._filters.discard(element)

    def filter(self, data: List[T]) -> List[T]:
        """TODO"""
        filtered: List[T] = []
        for element in data:
            if not self.should_filter(element):
                filtered.append(element)
        return filtered

    def filter_inverse(self, data: List[T]) -> List[T]:
        """TODO"""
        filtered: List[T] = []
        for element in data:
            if self.should_filter(element):
                filtered.append(element)
        return filtered

    def should_filter(self, data: T) -> bool:
        """TODO"""
        return not all(f.matches(data) for f in self._filters)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.factory
      @file: menu_entry.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from hspylib.modules.cli.tui.menu.menu import Menu
from hspylib.modules.cli.tui.menu.menu_item import MenuItem
from hspylib.modules.cli.vt100.vt_colors import VtColors
MENU_TPL = """%ED2%%HOM%
{}

{}
"""


class MenuEntry(MenuItem):
    """TODO"""

    def __init__(self, parent: Menu, items: dict, title: str = None, color: VtColors = VtColors.ORANGE):
        super().__init__(parent, title)
        title_len = round(len(title) / 2) + 2
        self.menu_data = MENU_TPL.format(
            f"{color.placeholder()}{'-=' * title_len}\n  {self.title}\n{'-=' * title_len}",
            '\n'.join([str(value) for key, value in items.items()])
        )
        self.options = range(0, len(items))
        self.items = items

    def trigger_menu_item(self) -> Menu:
        """TODO"""

        int_op = int(str(self.selected).strip())
        ret_val = self.items[str(int_op)].action_trigger(self)

        return ret_val if ret_val else self



# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.modules.cli.tui.extra
"""Package initialization."""

__all__ = [
  'mchoose', 
  'mdashboard', 
  'minput', 
  'mselect'
]
__version__ = '0.11.100'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: hconsole.py
   @created: Wed, 30 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from typing import Optional

from PyQt5.QtCore import QRect, QSize, Qt
from PyQt5.QtGui import QColor, QCursor, QFont, QPainter, QPaintEvent, QPalette, QResizeEvent, QTextCursor, QTextFormat
from PyQt5.QtWidgets import QPlainTextEdit, QTextEdit, QWidget


class HConsole(QPlainTextEdit):
    """TODO"""

    class _LineNumberArea(QWidget):
        """TODO"""

        def __init__(self, text_edit: 'HConsole'):
            super().__init__(text_edit)
            self._text_edit = text_edit

        def sizeHint(self) -> QSize:
            return QSize(self._text_edit.line_number_area_width(), 0)

        def paintEvent(self, event: QPaintEvent) -> None:
            self._text_edit.line_number_area_paint_event(event)

    def __init__(self, parent: Optional[QWidget], max_lines: int = 1000):
        super().__init__(parent)
        self._max_lines = max_lines

        # Defaults
        self._context_menu_enable = True
        self._clearable = True
        self._highlight_enabled = False
        self._show_line_numbers = False
        self.setReadOnly(True)
        self.setPlaceholderText('No messages received yet')
        self.setFont(QFont("Courier New", 14))
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self._context_menu)

        # Create line numbers widget
        self._line_number_area = self._LineNumberArea(self)
        self.document().blockCountChanged.connect(self.update_line_number_area_width)
        self.updateRequest.connect(self.update_line_number_area)
        self.cursorPositionChanged.connect(self.highlight_current_line)
        self.update_line_number_area_width()
        self.highlight_current_line()

    def resizeEvent(self, event: QResizeEvent) -> None:
        super().resizeEvent(event)
        cr = self.contentsRect()
        self._line_number_area.setGeometry(QRect(cr.left(), cr.top(), self.line_number_area_width(), cr.height()))

    def line_number_area_paint_event(self, event: QPaintEvent) -> None:
        """Paint the numbers of the line number area"""
        margin_right = -2
        bg_color = self.palette().color(QPalette.Window).lighter(100)
        txt_color = self.palette().color(QPalette.WindowText).darker(160)
        painter = QPainter(self._line_number_area)
        painter.fillRect(event.rect(), bg_color)
        block = self.firstVisibleBlock()
        block_number = block.blockNumber()
        top = round(self.blockBoundingGeometry(block).translated(self.contentOffset()).top())
        bottom = top + round(self.blockBoundingRect(block).height())
        while block.isValid() and top <= event.rect().bottom():
            if block.isVisible() and bottom >= event.rect().top():
                number = str(block_number + 1)
                painter.setPen(txt_color)
                painter.drawText(
                    margin_right, top, self._line_number_area.width(),
                    self.fontMetrics().height(), Qt.AlignRight, number)
            block = block.next()
            top = bottom
            bottom = top + round(self.blockBoundingRect(block).height())
            block_number += 1

    def line_number_area_width(self) -> int:
        """Return the current line number area width"""
        if self._show_line_numbers:
            digits = 1
            margin_left = 10
            max_count = max(1, self.document().blockCount())
            char_width = self.fontMetrics().widthChar('_')
            # Check the amount of digits to be printed
            while max_count >= 10:
                max_count /= 10
                digits += 1

            return margin_left + char_width * digits

        return 0

    def highlight_current_line(self) -> None:
        """Highlight current line number if highlighting is enabled"""
        if self._highlight_enabled and self.line_count() > 1:
            bg = self.palette().color(QPalette.Background).lighter(120)
            bg.setAlphaF(0.6)
            selection = QTextEdit.ExtraSelection()
            selection.format.setBackground(bg)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            self.setExtraSelections([selection])

    def update_line_number_area_width(self, new_block_count: int = 0) -> None:  # pylint: disable=unused-argument
        """Update the line number area width"""
        self.setViewportMargins(self.line_number_area_width(), 0, 0, 0)

    def update_line_number_area(self, rect: QRect, dy: int):
        """Update the line number area rect"""
        if dy:
            self._line_number_area.scroll(0, dy)
        else:
            self._line_number_area.update(0, rect.y(), self._line_number_area.width(), rect.height())
        if rect.contains(self.viewport().rect()):
            self.update_line_number_area_width()

    def line_count(self) -> int:
        """Return the number of lines the console contains"""
        return self.document().blockCount()

    def is_empty(self) -> bool:
        """Return true if console has no lines"""
        return self.line_count() == 0

    def set_plain_text(self, text: str) -> None:
        self.document().setPlainText(text)

    def set_html_text(self, text: str) -> None:
        self.document().setHtml(text)

    def push_text(self, text: str, color: QColor = None) -> None:
        """Push text to the console. If the maximum buffer size reached then lines are erased from the top"""
        fmt_text = f"<font color={color.name() if color else '#FFFFFF'}>{text}</font>"
        if self.line_count() + 1 > self._max_lines:
            self.pop_text()
        self.appendHtml(fmt_text)

    def pop_text(self, count: int = 0) -> str:
        """Pop <count> lines form the top->bottom"""
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.Start)
        cursor.movePosition(QTextCursor.Down, QTextCursor.MoveAnchor, count)
        cursor.select(QTextCursor.LineUnderCursor)
        selected_text = cursor.selectedText()
        cursor.removeSelectedText()
        cursor.deleteChar()
        self.setTextCursor(cursor)

        return selected_text

    def _context_menu(self) -> None:
        """Display the custom context menu"""
        if self._context_menu_enable:
            ctx_menu = self.createStandardContextMenu()
            if self._clearable:
                ctx_menu.addSeparator()
                ctx_menu.addAction('Clear', self.clear)
            ctx_menu.exec_(QCursor.pos())

    def set_context_menu_enable(self, enabled: bool = True):
        """Whether context menu is enabled or not"""
        self._context_menu_enable = enabled

    def set_clearable(self, clearable: bool = True):
        """Whether the widget is clearable or not"""
        self._clearable = clearable

    def set_highlight_enable(self, enabled: bool = True):
        """Whether line highlight is enabled or not"""
        self._highlight_enabled = enabled

    def set_show_line_numbers(self, show: bool = True):
        """Whether to show line numbers or not"""
        self._show_line_numbers = show
        self.update_line_number_area_width()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.application
      @file: version.py
   @created: Thu, 14 Feb 2022
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import logging as log
import os
from pathlib import Path
from typing import Tuple, Union

from hspylib.core.tools.commons import read_version


class Version:

    @staticmethod
    def load(filename: str = '.version', load_dir: Union[str, Path] = os.getcwd()) -> 'Version':
        filepath = f'{str(load_dir)}/{filename}'
        if not os.path.exists(filepath):
            log.warning('File "%s" does not exist. Could not fetch application version', filepath)
            return Version((0, 0, 0))

        return Version(read_version(filepath))

    def __init__(self, version: Tuple[int, int, int]):
        self.version = version
        self.version_string = '.'.join(map(str, version))

    def __str__(self):
        return self.version_string

    def __repr__(self):
        return str(self)

    def version(self) -> str:
        """TODO"""
        return f'v{str(self)}'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.icons.font_awesome
      @file: widget_icons.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome

class WidgetIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    WIDGET = '\uFC65'    # ﱥ
    DATABASE = '\uFB19'  # ﬙
    CLOCK = '\uF651'     # 
    NETWORK = '\uF819'   # 
    CHART_1 = '\uFC67'   # ﱨ
    CHART_2 = '\uFC68'   # ﱩ
    MUSIC = '\uFC6E'     # ﱯ
    SIGN = '\uFC7E'      # ﱿ
    SWORDS = '\uFC84'    # ﲅ
    PUNCH = '\uF255'     # 
    # @formatter:on


if __name__ == '__main__':
    WidgetIcons.demo_icons()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: main.enum
      @file: enumeration.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
from enum import Enum
from typing import Any, List, TypeVar

from hspylib.core.tools.preconditions import check_not_none

E = TypeVar('E', bound='Enumeration')


class Enumeration(Enum):
    """TODO"""

    @classmethod
    def names(cls) -> List[str]:
        """TODO"""
        return list(map(lambda e: e.name, cls))

    @classmethod
    def values(cls) -> List[Any]:
        """TODO"""
        return list(map(lambda e: e.value, cls))

    @classmethod
    def value_of(cls, name: str, ignore_case: bool = True) -> E:
        """TODO"""
        if ignore_case:
            found = next(filter(lambda en: en.name.upper() == name.upper(), list(cls)), None)
        else:
            found = next(filter(lambda en: en.name == name, list(cls)), None)
        return check_not_none(found, "\"{}\" name does not correspond to a valid \"{}\" enum", name, cls.__name__)

    @classmethod
    def of_value(cls, value: Any, ignore_case: bool = False) -> E:
        if ignore_case:
            found = next(filter(lambda en: str(en.value).upper() == str(value).upper(), list(cls)), None)
        else:
            found = next(filter(lambda en: en.value == value, list(cls)), None)
        return check_not_none(found, "\"{}\" value does not correspond to a valid \"{}\" enum", value, cls.__name__)

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return str(self)

    def __eq__(self, other: E) -> bool:
        return (
            self.__class__.__qualname__ == other.__class__.__qualname__ and
            (other.name == self.name and self.value == other.value)
        )

    def __hash__(self) -> int:
        hash_code = 7
        hash_code = 31 * hash_code + (0 if self.name is None else hash(self.name))
        hash_code = 31 * hash_code + (0 if self.value is None else hash(self.value))
        return hash_code



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.icons.font_awesome
      @file: webapp_icons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
from hspylib.modules.cli.icons.font_awesome.awesome import Awesome

class WebappIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    GITHUB = '\uF408'           # 
    TWITTER = '\uF081'          # 
    FACEBOOK = '\uF082'         # 
    LINKED_IN = '\uF08C'        # 
    STACK_OVERFLOW = '\uF9CB'   # 溜
    DROPBOX = '\uFC29'          # ﰩ
    # @formatter:on


if __name__ == '__main__':
    WebappIcons.demo_icons()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   main.addons.widman
      @file: widget_entry.py
   @created: Fri, 04 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import os

from hspylib.core.tools.text_tools import camelcase


class WidgetEntry:
    """Placeholder object for a HSPyLib::Widman Widget and details about it."""

    MODULE_PREFIX = 'widget_'
    CLASS_PREFIX = 'Widget'

    def __init__(self, file: str, path: str):
        self.module = os.path.splitext(file)[0]
        self.name = camelcase(self.module.replace(self.MODULE_PREFIX, ''), upper=True)
        self.clazz = f"{self.CLASS_PREFIX}{self.name.replace('_', '')}"
        self.path = path

    def __str__(self):
        return f"{self.name}: {self.module}.{self.clazz} => {self.path}"

    def __repr__(self):
        return str(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: main.crud.db.mysql
      @file: mysql_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import logging as log
import uuid
from abc import abstractmethod
from typing import List, Optional, Tuple

import pymysql
from pymysql.err import OperationalError, ProgrammingError
from requests.structures import CaseInsensitiveDict as SqlFilter

from hspylib.core.crud.crud_entity import CrudEntity
from hspylib.core.crud.db.db_repository import DBRepository
from hspylib.core.crud.db.sql_factory import SqlFactory
from hspylib.core.exception.exceptions import NotConnectedError


class MySqlRepository(DBRepository):
    """Implementation of a data access layer for a MySql persistence store."""

    _connection_cache = {}

    def __init__(self):
        super().__init__()
        self._connector = None
        self._cursor = None
        self._sql_factory = SqlFactory()

    def is_connected(self) -> bool:
        """Whether connected or not to a MySql database."""
        return self._connector is not None

    def connect(self) -> None:
        """Attempts to connect to a MySql database."""
        if not self.is_connected():
            cache_key = str(self)
            if cache_key in self._connection_cache:
                self._connector = self._connection_cache[cache_key]
                self._cursor = self._connector.cursor()
                if not self.is_connected():
                    raise ConnectionError("Not connected to the database")
            else:
                db_url = f"{self.user}@{self.hostname}:{self.port}/{self.database}"
                try:
                    self._connector = pymysql.connect(
                        host=self.hostname,
                        user=self.user,
                        port=self.port,
                        password=self.password,
                        database=self.database)
                    if not self.is_connected():
                        raise ConnectionError(f"Unable to connect to {db_url}")
                    self._cursor = self._connector.cursor()
                    log.debug("Connection to %s established", db_url)
                    self._connection_cache[cache_key] = self._connector
                except (OperationalError, ConnectionRefusedError) as err:
                    raise ConnectionError(f"Unable to connect to {db_url}") from err

    def disconnect(self) -> None:
        """Attempts to disconnect from a MySql database."""
        if self.is_connected():
            cache_key = str(self)
            self._connector.close()
            del self._connector
            del self._connection_cache[cache_key]
            log.debug('Disconnected from %s', str(self))
        else:
            raise NotConnectedError('Not connected to a MySql database.')

    def insert(self, entity: CrudEntity) -> None:
        """Saves the given entity at the MySql store"""
        if self.is_connected():
            entity.uuid = entity.uuid if entity.uuid is not None else str(uuid.uuid4())
            stm = self._sql_factory \
                .insert(entity) \
                .replace(':tableName', self.table_name())
            log.debug("Executing SQL statement: %s", stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to a MySql database.')

    def update(self, entity: CrudEntity) -> None:
        """Updates the given entity at the MySql store"""
        if self.is_connected():
            stm = self._sql_factory \
                .update(entity, filters=SqlFilter({"UUID": f'{entity.uuid}'})) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to a MySql database.')

    def delete(self, entity: CrudEntity) -> None:
        """Deletes the given entity at the MySql store"""
        if self.is_connected():
            stm = self._sql_factory \
                .delete(filters=SqlFilter({"UUID": f'{entity.uuid}'})) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to a MySql database.')

    def find_all_columns(
        self,
        column_set: List[str] = None,
        filters: SqlFilter = None) -> Optional[List[CrudEntity]]:
        """TODO"""

        if self.is_connected():
            stm = self._sql_factory \
                .select(column_set=column_set, filters=filters) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
            result = self._cursor.fetchall()
            return list(map(self.row_to_entity, result)) if result else None

        raise NotConnectedError('Not connected to a MySql database.')

    def find_columns_by_id(
        self,
        column_set: List[str] = None,
        entity_id: str = None) -> Optional[CrudEntity]:
        """TODO"""

        if self.is_connected():
            if entity_id:
                stm = self._sql_factory \
                    .select(column_set=column_set, filters=SqlFilter({"UUID": f'{entity_id}'})) \
                    .replace(':tableName', self.table_name())
                log.debug('Executing SQL statement: %s', stm)
                self.execute(stm, True)
                result = self._cursor.fetchall()
                if len(result) > 1:
                    raise ProgrammingError(f'Multiple results found {len(result)}')
                return self.row_to_entity(result[0]) if len(result) > 0 else None
            return None

        raise NotConnectedError('Not connected to a MySql database.')

    def execute(self, sql_statement: str, auto_commit: bool, *params) -> None:
        """TODO"""
        if self.is_connected():
            log.debug("Executing SQL statement: %s with params: [%s]", sql_statement, ', '.join(params))
            self._cursor.execute(sql_statement, *params)
            if auto_commit:
                self.commit()
        else:
            raise NotConnectedError('Not connected to a MySql database.')

    def commit(self) -> None:
        """TODO"""
        log.debug('Committing database changes')
        self._connector.commit()

    def rollback(self) -> None:
        """TODO"""
        log.debug('Rolling back database changes')
        self._connector.rollback()

    @abstractmethod
    def row_to_entity(self, row: Tuple) -> CrudEntity:
        pass

    @abstractmethod
    def table_name(self) -> str:
        pass



# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.core.crud.db.firebase
"""Package initialization."""

__all__ = [
  'firebase_config', 
  'firebase_repository'
]
__version__ = '0.11.100'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.icons.emojis.faces
      @file: face_smiling.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from hspylib.modules.cli.icons.emojis.emojis import emj_print, Emoji

class FaceSmiling(Emoji):
    """
        Face smiling emojis.
        Codes can be found here:
        - https://unicode.org/emoji/charts/emoji-list.html#face-smiling
    """
    DEFAULT = '\U0001F600'
    BEAMING = '\U0001F601'
    TEARS_OF_JOY = '\U0001F602'
    BIG_EYES = '\U0001F603'
    SMILING_EYES = '\U0001F604'
    SWEAT = '\U0001F605'
    SQUINTING = '\U0001F606'
    HALO = '\U0001F607'
    WINKING = '\U0001F609'
    BLUSHING = '\U0001F60A'
    SLIGHTLY = '\U0001F642'
    UPSIDE_DOWN = '\U0001F643'
    ROFL = '\U0001F923'

    @classmethod
    def demo_emojis(cls) -> bool:
        list(map(emj_print, cls.values()))


if __name__ == '__main__':
    FaceSmiling.demo_emojis()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.metaclass
      @file: singleton.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import logging as log
import traceback
from typing import Type

from hspylib.core.tools.preconditions import check_not_none


# pylint: disable=bad-mcs-classmethod-argument


class Singleton(type):
    """TODO"""

    _instances = {}

    def __call__(cls, *args, **kwargs):
        """TODO"""
        if not cls.has_instance(cls):
            try:
                instance = super(Singleton, cls).__call__(*args, **kwargs)
                check_not_none(instance, f'Unable to create Singleton instance: {cls}')
                cls.INSTANCE = instance
                cls._instances[cls] = instance
                log.debug('Created a new Singleton instance: %s.%s', cls.__module__, cls.__name__)
            except Exception as err:  # pylint: disable=broad-except
                log.error(traceback.format_exc())
                raise err from err

        return cls._instances[cls]

    @classmethod
    def instances(mcs) -> dict:
        """TODO"""
        return Singleton._instances

    @classmethod
    def has_instance(mcs, cls) -> bool:
        """TODO"""
        return cls in Singleton._instances

    @classmethod
    def del_instance(mcs, clazz: Type) -> bool:
        """TODO"""
        singleton_instances = Singleton.__getattribute__(mcs, '_instances')
        if clazz in singleton_instances:
            log.warning('Deleted an existing Singleton instance: %s.%s', mcs.__module__, mcs.__name__)
            del mcs._instances[clazz]
            delattr(clazz, 'INSTANCE')
            del clazz
            return True

        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: hspylib
   @package: hspylib.main.hspylib
      @file: __classpath__.py
   @created: Wed, 8 Jun 2022
    @author: "<B>H</B>ugo <B>S</B>aporetti <B>J</B>unior")"
      @site: "https://github.com/yorevs/hspylib")
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import os
from pathlib import Path
from textwrap import dedent
from typing import Optional, Union

from hspylib.core.exception.exceptions import ResourceNotFoundError, SourceNotFoundError
from hspylib.core.metaclass.singleton import Singleton
from hspylib.core.tools.preconditions import check_state


class Classpath(metaclass=Singleton):
    """TODO"""

    def __init__(self, source_root: Path, run_dir: Path, resource_dir: Optional[Path]):
        check_state(
            source_root.exists()
            and run_dir.exists(),
            "source_root / run_dir are mandatory and must exist")
        self.source_root = source_root
        self.resource_dir = resource_dir
        self.run_dir = run_dir
        self.log_dir = os.getenv('LOG_DIR', self.run_dir)

    @classmethod
    def list_resources(cls, directory: Union[Path, str]) -> str:
        """TODO"""
        res_str = cls.list_files(directory)
        for root, dirs, _ in os.walk(directory):
            for dirname in dirs:
                res_str += '  |-' + dirname + os.linesep
                res_str += cls.list_resources(os.path.join(root, dirname))
        return res_str

    @classmethod
    def list_files(cls, directory: Union[Path, str]) -> str:
        """TODO"""
        res_str = ''
        for filename in os.listdir(directory):
            f = os.path.join(directory, filename)
            if os.path.isfile(f):
                res_str += '    |-' + str(filename) + os.linesep
        return res_str

    @classmethod
    def source_root(cls) -> Path:
        """TODO"""
        return cls.INSTANCE.source_root

    @classmethod
    def run_dir(cls) -> Path:
        """TODO"""
        return cls.INSTANCE.run_dir

    @classmethod
    def resource_dir(cls) -> Path:
        """TODO"""
        return cls.INSTANCE.resource_dir

    @classmethod
    def log_dir(cls) -> Optional[Path]:
        """TODO"""
        return cls.INSTANCE.log_dir

    @classmethod
    def get_resource(cls, resource_path: Union[str, Path]) -> Path:
        """TODO"""
        resource = Path(f'{cls.INSTANCE.resource_dir}/{str(resource_path)}')
        if not resource.exists():
            raise ResourceNotFoundError(f'Resource {str(resource_path)} was not found!')
        return resource

    @classmethod
    def get_source(cls, source_path: Union[str, Path]) -> Path:
        """TODO"""
        source_path = Path(f'{cls.INSTANCE.source_root}/{str(source_path)}')
        if not source_path.exists():
            raise SourceNotFoundError(f'Source {str(source_path)} was not found!')
        return source_path

    def __str__(self):
        return dedent(f"""
        |-run-dir: {self.run_dir}
        |-source-root: {self.source_root}
        |-resources: {self.resource_dir}
        |-log_dir: {self.log_dir}
        """) + self.list_resources(self.resource_dir)

    def __repr__(self):
        return str(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   main.addons.widman
      @file: widman.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import atexit
import os
import sys
from typing import List

from hspylib.addons.widman.widget import Widget
from hspylib.addons.widman.widget_entry import WidgetEntry
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.exception.exceptions import WidgetExecutionError, WidgetNotFoundError
from hspylib.core.metaclass.singleton import Singleton
from hspylib.core.tools.commons import get_path
from hspylib.core.tools.preconditions import check_state
from hspylib.core.tools.text_tools import camelcase
from hspylib.modules.cli.application.application import Application
from hspylib.modules.cli.tui.extra.mdashboard.dashboard_item import DashboardItem
from hspylib.modules.cli.tui.extra.mdashboard.mdashboard import mdashboard

HERE = get_path(__file__)


class WidgetManager(metaclass=Singleton):
    """HSPyLib widget manager that handles HSPyLib widgets"""

    WIDGETS_PATH = (HERE / "widgets")

    @staticmethod
    def _name_matches(widget_1_name: str, widget_2_name: str) -> bool:
        """Check if two names matches using defined naming rules"""

        return \
            widget_1_name.lower() == widget_2_name.lower() \
            or widget_1_name == widget_2_name.capitalize() \
            or widget_1_name == camelcase(widget_2_name, upper=True) \
            or widget_1_name.lower() == widget_2_name.lower().replace('_', '')

    def __init__(self, parent_app: Application):
        self._parent_app = parent_app
        self._widgets = []
        self._lookup_paths = os.environ.get('HHS_WIDGETS_PATH', '').split(':')
        self._lookup_paths.insert(0, str(WidgetManager.WIDGETS_PATH))
        list(map(sys.path.append, self._lookup_paths))
        check_state(self._load_widgets() > 0, "Unable to find any widgets from: {}", self._lookup_paths)

    def execute(self, widget_name: str, widget_args: List[str]) -> None:
        """Execute the specified widget"""

        widget = self._find_widget(camelcase(widget_name, upper=True))
        try:
            atexit.register(widget.cleanup)
            exit_code = widget.execute(widget_args)
            if exit_code in [ExitCode.ERROR, ExitCode.FAILED]:
                raise WidgetExecutionError(f"Widget '{widget_name}' failed to execute. exit_code={exit_code}")
        except Exception as err:
            raise WidgetExecutionError(f"Unable to execute widget '{widget_name}' -> {err}") from err

    def dashboard(self) -> None:
        """Display all available widgets from the widget lookup paths"""

        items = []
        try:
            for widget_entry in self._widgets:
                widget = self._find_widget(widget_entry.name)
                item = DashboardItem(
                    widget.icon(), f"{widget.name()} v{widget.version()}: {widget.tooltip()}", widget.execute)
                items.append(item)
            check_state(len(items) > 0, "No widgets found from: {}", str(self._lookup_paths))
            mdashboard(items, 6, 'Please select a widget to execute')
        except Exception as err:
            raise WidgetExecutionError(f"Failed to execute widget :: {str(err)}") from err

    # pylint: disable=cell-var-from-loop
    def _load_widgets(self) -> int:
        """Search and load all widgets from the widget lookup paths"""

        for path in self._lookup_paths:
            for root, _, files in os.walk(path):
                filtered = list(filter(
                    lambda p: p.startswith(WidgetEntry.MODULE_PREFIX) and p.endswith('py'), files
                ))
                widgets = list(map(lambda w: WidgetEntry(w, f"{root}/{w}"), filtered))
                self._widgets.extend(widgets)

        return len(self._widgets)

    def _find_widget(self, widget_name: str) -> Widget:
        """
        Find and return a widget specified by 'widget_name'. If no widget is found,
        then, an exception will be raised
        """

        widget_entry = next((w for w in self._widgets if self._name_matches(widget_name, w.name)), None)
        if not widget_entry:
            raise WidgetNotFoundError(
                f"Widget '{widget_name}' was not found on widget lookup paths: {str(self._lookup_paths)}")
        try:
            widget_module = __import__(widget_entry.module)
        except ModuleNotFoundError as err:
            raise WidgetNotFoundError(
                f"Widget '{widget_name}' was not found on widget lookup paths: {str(self._lookup_paths)}") from err
        widget_clazz = getattr(widget_module, widget_entry.clazz)
        widget = widget_clazz()
        check_state(isinstance(widget, Widget), 'All widgets must inherit from "addons.widman.widget.Widget"')

        return widget



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.tools
      @file: preconditions.py
   @created: Fri, 16 Jul 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from typing import Any, Optional, Tuple, TypeVar, Union

from hspylib.core.exception.exceptions import InvalidArgumentError, InvalidStateError

T = TypeVar('T')


def check_argument(expression: bool, error_message: str = None, *args) -> bool:
    """Ensures the truth of an expression involving one or more parameters to the calling method."""
    if not expression:
        raise InvalidArgumentError(
            error_message.format(*args) if error_message else 'Precondition failed: Invalid argument')

    return expression


def check_state(expression: bool, error_message: str = None, *args) -> bool:
    """Ensures the truth of an expression involving the state of the calling instance, but not involving
    any parameters to the calling method."""
    if not expression:
        raise InvalidStateError(
            error_message.format(*args) if error_message else 'Precondition failed: Invalid state')

    return expression


def check_not_none(references: Union[T, Tuple[T]], error_message: str = None, *args) -> T:
    """Ensures that an object reference passed as a parameter to the calling method is not None."""
    if isinstance(references, Tuple):
        if not all(ref is not None for ref in references):
            raise TypeError(
                error_message.format(*args) if error_message else 'Precondition failed: <None> reference found')
    if references is None:
        raise TypeError(
            error_message.format(*args) if error_message else 'Precondition failed: <None> reference found')
    return references


def check_element_index(index: int, array: list, desc: str = None) -> int:
    """Ensures that index specifies a valid element in an array, list or string of size size."""

    size = len(array)
    if size < 0:
        raise InvalidArgumentError('Size is negative')
    if index < 0 or index >= size:
        raise IndexError(desc or 'Precondition failed: Index is negative or out of bounds')

    return index


def check_index_in_range(start: int, end: int, array: list, desc: str = None) -> Tuple[int, int]:
    """Ensures that start and end specify a valid positions in an array, list or string of size size, and
    are in order."""

    size = len(array)
    if size < 0:
        raise InvalidArgumentError('Size is negative')
    if start < 0 or end < 0 or start >= size or end >= size:
        raise IndexError(desc or 'Precondition failed: Index is negative or greater than size')
    if end < start:
        raise IndexError(desc or 'Precondition failed: End is less than start')

    return start, end


def check_and_get(
    element_name: str,
    content_dict: dict = None,
    required: bool = True,
    default: Any = None) -> Optional[Any]:
    """Ensures that the element is in the content dictionary. If it is required and not found, an exception will be
     raised. If it is not found and not required, the default value is returned. If the element is found, then, it's a
     value is returned."""

    if content_dict and element_name in content_dict:
        return content_dict[element_name]

    if required:
        raise InvalidArgumentError(
            f'Precondition failed: Required attribute {element_name} was not found in the content dictionary')

    return default



# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib
"""Package initialization."""

__all__ = [
  '__classpath__', 
  '__main__', 
  'addons', 
  'core', 
  'modules'
]
__version__ = '0.11.100'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.extra.minput
      @file: form_builder.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from typing import Any

from hspylib.modules.cli.tui.extra.minput.field_builder import FieldBuilder


class FormBuilder:
    """TODO"""

    def __init__(self):
        self.fields = []

    def field(self) -> Any:
        return FieldBuilder(self)

    def build(self) -> list:
        return self.fields



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file

   @project: HSPyLib
   @package: main.addons.widman.widgets
      @file: widget_punch.py
   @created: Thu, 20 Sep 2022
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""
import os
from datetime import datetime
from textwrap import dedent
from typing import List

from hspylib.addons.widman.widget import Widget
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.icons.font_awesome.widget_icons import WidgetIcons


class WidgetPunch(Widget):
    """HSPyLib Widget to Report current system memory usage"""

    WIDGET_ICON = WidgetIcons.PUNCH
    WIDGET_NAME = "Punch"
    TOOLTIP = "!!! PUNCH THE CLOCK !!!"
    USAGE = dedent("""
    "Usage: ${FUNCNAME[0]} [options] <args>"

      Options: '
        -l        : List all registered punches.'
        -e        : Edit current punch file.'
        -r        : Reset punches for the current week and save the previous one.'
        -w <week> : Report (list) all punches of specified week using the pattern: week-N.punch.'

      Notes: '
        When no arguments are provided it will !!PUNCH THE CLOCK!!.'
    """)
    VERSION = (0, 1, 0)

    HHS_DIR = os.getenv('HHS_PUNCH_FILE', os.getenv('HOME', './'))

    HHS_PUNCH_FILE = os.getenv('HHS_PUNCH_FILE', f"-{HHS_DIR}/.punches")

    DATE_STAMP='%a %d-%m-%Y'

    TIME_STAMP='%H:%M'

    WEEK_STAMP='%V'

    def __init__(self):
        super().__init__(
            self.WIDGET_ICON,
            self.WIDGET_NAME,
            self.TOOLTIP,
            self.USAGE,
            self.VERSION)

        self._exit_code = ExitCode.SUCCESS

    def execute(self, args: List[str] = None) -> ExitCode:

        if args and args[0] in ['-h', '--help']:
            sysout(self.usage())
            return ExitCode.SUCCESS
        if args and args[0] in ['-v', '--version']:
            sysout(self.version())
            return ExitCode.SUCCESS

        date_stamp = datetime.today().strftime('%d-%m-%Y')

        # Create the punch file if it does not exist
        if not os.path.exists(self.HHS_PUNCH_FILE):
            with open(self.HHS_PUNCH_FILE, 'w') as f_punch:
                f_punch.write(f"{date_stamp} => ")

        print('Punch')

        return self._exit_code




# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.modules.cli.vt100
"""Package initialization."""

__all__ = [
  'terminal', 
  'vt_100', 
  'vt_codes', 
  'vt_colors', 
  'vt_utils'
]
__version__ = '0.11.100'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.icons.emojis
      @file: emojis.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

from emoji.core import emojize

from hspylib.core.enums.enumeration import Enumeration


def emj_print(emoji_str: str) -> None:
    print(emojize(emoji_str) + ' ', end='')


class Emoji(Enumeration):
    """
        Emoji codes
        Full list of emojis can be found here:
          - https://unicode.org/emoji/charts/emoji-list.html
    """

    def __str__(self) -> str:
        return str(self.value)

    def placeholder(self) -> str:
        return f":{self.name}:"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.tools
      @file: keyboard.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2022, HSPyLib team
"""

import select
import string
import sys
from typing import Optional

import getkey

from hspylib.core.enums.enumeration import Enumeration
from hspylib.core.exception.exceptions import KeyboardInputError
from hspylib.modules.cli.vt100.vt_utils import require_terminal

require_terminal()


class Keyboard(Enumeration):
    """TODO"""

    # Control keys
    VK_NONE = ''
    VK_ESC = getkey.keys.ESC
    VK_UP = getkey.keys.UP
    VK_DOWN = getkey.keys.DOWN
    VK_LEFT = getkey.keys.LEFT
    VK_RIGHT = getkey.keys.RIGHT
    VK_BACKSPACE = getkey.keys.BACKSPACE
    VK_INSERT = getkey.keys.INSERT
    VK_DELETE = getkey.keys.DELETE
    VK_ENTER = getkey.keys.ENTER
    VK_TAB = '\t'
    VK_SPACE = getkey.keys.SPACE
    VK_HOME = getkey.keys.HOME
    VK_END = getkey.keys.END
    VK_PAGE_UP = getkey.keys.PAGE_UP
    VK_PAGE_DOWN = getkey.keys.PAGE_DOWN
    VK_SHIFT_TAB = '\x1b[Z'
    # Letters
    VK_a = 'a'
    VK_A = 'A'
    VK_b = 'b'
    VK_B = 'B'
    VK_c = 'c'
    VK_C = 'C'
    VK_d = 'd'
    VK_D = 'D'
    VK_e = 'e'
    VK_E = 'E'
    VK_f = 'f'
    VK_F = 'F'
    VK_g = 'g'
    VK_G = 'G'
    VK_h = 'h'
    VK_H = 'H'
    VK_i = 'i'
    VK_I = 'I'
    VK_j = 'j'
    VK_J = 'J'
    VK_k = 'k'
    VK_K = 'K'
    VK_l = 'l'
    VK_L = 'L'
    VK_m = 'm'
    VK_M = 'M'
    VK_n = 'n'
    VK_N = 'N'
    VK_o = 'o'
    VK_O = 'O'
    VK_p = 'p'
    VK_P = 'P'
    VK_q = 'q'
    VK_Q = 'Q'
    VK_r = 'r'
    VK_R = 'R'
    VK_s = 's'
    VK_S = 'S'
    VK_t = 't'
    VK_T = 'T'
    VK_u = 'u'
    VK_U = 'U'
    VK_v = 'v'
    VK_V = 'V'
    VK_w = 'w'
    VK_W = 'W'
    VK_x = 'x'
    VK_X = 'X'
    VK_y = 'y'
    VK_Y = 'Y'
    VK_z = 'z'
    VK_Z = 'Z'
    # Numbers
    VK_ZERO = '0'
    VK_ONE = '1'
    VK_TWO = '2'
    VK_THREE = '3'
    VK_FOUR = '4'
    VK_FIVE = '5'
    VK_SIX = '6'
    VK_SEVEN = '7'
    VK_EIGHT = '8'
    VK_NINE = '9'
    # Punctuation
    VK_BACKTICK = '`'
    VK_TILDE = '~'
    VK_EXCLAMATION_MARK = '!'
    VK_AT = '@'
    VK_SHARP = '#'
    VK_DOLLAR = '$'
    VK_PERCENTAGE = '%'
    VK_CIRCUMFLEX = '^'
    VK_AMPERSAND = '&'
    VK_ASTERISK = '*'
    VK_OPEN_PARENTHESIS = '('
    VK_CLOSE_PARENTHESIS = ')'
    VK_MINUS = '-'
    VK_UNDERSCORE = '_'
    VK_PLUS = '+'
    VK_EQUALS = '='
    VK_LEFT_BRACKET = '['
    VK_RIGHT_BRACKET = ']'
    VK_LEFT_BRACE = '{'
    VK_RIGHT_BRACE = '}'
    VK_PIPE = '|'
    VK_BACK_SLASH = '\\'
    VK_BACK_COLON = ':'
    VK_BACK_SEMI_COLON = ';'
    VK_QUOTE = '\''
    VK_DOUBLE_QUOTE = '"'
    VK_LOWER = '<'
    VK_GREATER = '>'
    VK_COMMA = ','
    VK_PERIOD = '.'
    VK_SLASH = '/'
    VK_QUESTION_MARK = '?'

    @staticmethod
    def kbhit() -> bool:
        """TODO"""
        dr, _, _ = select.select([sys.stdin], [], [], 0)
        return dr != []

    @staticmethod
    def getch() -> str:
        """TODO"""
        return sys.stdin.read(1)

    @classmethod
    def read_keystroke(cls, blocking: bool = True, ignore_error_keys: bool = True) -> Optional['Keyboard']:
        """TODO"""
        try:
            keystroke = getkey.getkey(blocking)
            if keystroke:
                return cls.of_value(keystroke)
        except (KeyboardInterrupt, AssertionError) as err:
            if not ignore_error_keys:
                raise KeyboardInputError(f"Invalid keystroke => {str(err)}") from err
        finally:
            sys.stdin.flush()

        return None

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return str(self)

    def isdigit(self) -> bool:
        """TODO"""
        return str(self.value).isdigit()

    def isalpha(self) -> bool:
        """TODO"""
        return str(self.value).isalpha()

    def isalnum(self) -> bool:
        """TODO"""
        return str(self.value).isalnum()

    def ispunct(self) -> bool:
        """TODO"""
        return all(ch in string.punctuation for ch in str(self.value))
